﻿using DocuSignAPI.APIService;
using EnvelopeCreatorProject.Models.Recipients.IdentityCheck;
using System.Collections.Generic;
using System.Text;

namespace EnvelopeCreatorProject.Models.Recipients
{
    internal abstract class RecipientType
    {
        protected StringBuilder _recipients;
        protected int _counter;
        protected int _totalCount;

        internal RecipientType(int totalCount)
        {
            _counter = 0;
            _recipients = new StringBuilder();
            _totalCount = totalCount;
        }

        protected abstract void AddData(Recipient objRecipient);
        
        internal void AddRecipientData(Recipient objRecipient) {
            _counter++;
            _recipients.AppendLine("{");
            AddData(objRecipient);
            _recipients.AppendLine("}");
            if (_counter < _totalCount) _recipients.AppendLine(",");
        }

        protected void AddRecipientCommonInfo(Recipient objRecipient)
        {
            _recipients.AppendLine("\"recipientId\":\"" + objRecipient.ID + "\",");
            _recipients.AppendLine("\"accessCode\":\"" + objRecipient.AccessCode + "\",");

            _recipients.AppendLine(GetIdentityCheckBuilder(objRecipient.IDCheckConfigurationName).Build(objRecipient));

            _recipients.AppendLine("\"routingOrder\":\"" + objRecipient.RoutingOrder + "\"");

        }

        private IIdentityCheck GetIdentityCheckBuilder(string key)
        {
            
            var _identityBuilders = new Dictionary<string, IIdentityCheck>() { 
                {"", new NoAuthType()},
                {"ID Check $", new IDCheck()},
                {"SMS Auth $", new SMSAuthType()},
                {"Phone Auth $", new PhoneAuthType()},
                {"Live ID", new SocialAuthType()},
                {"Facebook", new SocialAuthType()},
                {"Linkedin", new SocialAuthType()},
                {"Google", new SocialAuthType()},
                {"Salesforce", new SocialAuthType()},
                {"Twitter", new SocialAuthType()},
                {"Yahoo", new SocialAuthType()},
                {"OpenID", new SocialAuthType()}
            };

            if (string.IsNullOrEmpty(key)) return new NoAuthType();
            return (_identityBuilders.ContainsKey(key)) ? _identityBuilders[key] : new NoAuthType();
        }

        internal bool HasData() { return _recipients.Length > 0; }
        internal virtual string GetRecipientsData() { return _recipients.ToString(); }
    }
}
